<?php
// Create Full-Text Feeds
// Author: Keyvan Minoukadeh
// License: AGPLv3
// Version: 1.0
// Date: 2010-03-05

/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Usage
// -----
// Request this file passing it your feed in the querystring: makefulltextfeed.php?url=http://mysite.org
// The following options can be passed in the querystring:
// * URL: url=[feed or website url] (required)
// * Treat URL as HTML (not feed): html=true (optional, by default it's automatically detected)
// * API key: key=[api key] (optional)
// * Max entries to process (only with API key): max=[max number of items] (optional)

error_reporting(E_ALL ^ E_NOTICE);
ini_set("display_errors", 1);
@set_time_limit(120);

// Include SimplePie for RSS/Atom parsing
require_once('libraries/simplepie/simplepie.inc');
// Include FeedCreator for RSS/Atom creation
//require_once('libraries/feedcreator/include/feedcreator.class.php');
require_once('libraries/feedwriter/FeedWriter.php');
require_once('libraries/feedwriter/FeedItem.php');
// Include readability.php for identifying and extracting content from URLs
require_once('readability.php');

////////////////////////////////
// Load config file if it exists
////////////////////////////////
// the config values below should be set in config.php (rename config-sample.php if config.php doesn't exist).
// the values below will only be used if config.php doesn't exist.
$options->enabled = true;
$options->api_keys = array();
$options->max_entries = 4;
$options->max_entries_with_key = 6;
$options->message_to_prepend = '';
$options->message_to_prepend_with_key = '';
$options->message_to_append = '';
$options->message_to_append_with_key = '';
$options->blocked_urls = array();
$options->alternative_url = '';
$options->error_message = '[unable to retrieve full-text content]';
$options->error_message_with_key = '[unable to retrieve full-text content]';
if (file_exists(dirname(__FILE__).'/config.php')) {
	require_once(dirname(__FILE__).'/config.php');
}

//////////////////////////////////////////////
// Convert $html to UTF8
// (uses HTTP headers and HTML to find encoding)
// adapted from http://stackoverflow.com/questions/910793/php-detect-encoding-and-make-everything-utf-8
//////////////////////////////////////////////
function convert_to_utf8($html, $header=null)
{
	$accept = array(
		'type' => array('application/rss+xml', 'application/xml', 'application/rdf+xml', 'text/xml'),
		'charset' => array_diff(mb_list_encodings(), array('pass', 'auto', 'wchar', 'byte2be', 'byte2le', 'byte4be', 'byte4le', 'BASE64', 'UUENCODE', 'HTML-ENTITIES', 'Quoted-Printable', '7bit', '8bit'))
	);
	$encoding = null;
	if ($html || $header) {
		if (is_array($header)) $header = implode("\n", $header);
		if (!$header || !preg_match_all('/^Content-Type:\s+([^;]+)(?:;\s*charset=(.*))?/im', $header, $match, PREG_SET_ORDER)) {
			// error parsing the response
		} else {
			$match = end($match); // get last matched element (in case of redirects)
			if (!in_array(strtolower($match[1]), $accept['type'])) {
				// type not accepted
				// TODO: avoid conversion
			}
			if (isset($match[2])) $encoding = trim($match[2], '"\'');
		}
		if (!$encoding) {
			if (preg_match('/^<\?xml\s+version=(?:"[^"]*"|\'[^\']*\')\s+encoding=("[^"]*"|\'[^\']*\')/s', $html, $match)) {
				$encoding = trim($match[1], '"\'');
			} elseif(preg_match('/<meta\s+http-equiv="Content-Type" content="([^;]+)(?:;\s*charset=(.*))?"/i', $html, $match)) {
				if (isset($match[2])) $encoding = trim($match[2], '"\'');
			}
		}
		if (!$encoding) {
			$encoding = 'utf-8';
		} else {
			if (!in_array($encoding, array_map('strtolower', $accept['charset']))) {
				// encoding not accepted
				// TODO: avoid conversion
			}
			if ($encoding != 'utf-8') {
				if (strtolower($encoding) == 'iso-8859-1') {
					// replace MS Word smart qutoes
					$trans = array();
					$trans[chr(130)] = '&sbquo;';    // Single Low-9 Quotation Mark
					$trans[chr(131)] = '&fnof;';    // Latin Small Letter F With Hook
					$trans[chr(132)] = '&bdquo;';    // Double Low-9 Quotation Mark
					$trans[chr(133)] = '&hellip;';    // Horizontal Ellipsis
					$trans[chr(134)] = '&dagger;';    // Dagger
					$trans[chr(135)] = '&Dagger;';    // Double Dagger
					$trans[chr(136)] = '&circ;';    // Modifier Letter Circumflex Accent
					$trans[chr(137)] = '&permil;';    // Per Mille Sign
					$trans[chr(138)] = '&Scaron;';    // Latin Capital Letter S With Caron
					$trans[chr(139)] = '&lsaquo;';    // Single Left-Pointing Angle Quotation Mark
					$trans[chr(140)] = '&OElig;    ';    // Latin Capital Ligature OE
					$trans[chr(145)] = '&lsquo;';    // Left Single Quotation Mark
					$trans[chr(146)] = '&rsquo;';    // Right Single Quotation Mark
					$trans[chr(147)] = '&ldquo;';    // Left Double Quotation Mark
					$trans[chr(148)] = '&rdquo;';    // Right Double Quotation Mark
					$trans[chr(149)] = '&bull;';    // Bullet
					$trans[chr(150)] = '&ndash;';    // En Dash
					$trans[chr(151)] = '&mdash;';    // Em Dash
					$trans[chr(152)] = '&tilde;';    // Small Tilde
					$trans[chr(153)] = '&trade;';    // Trade Mark Sign
					$trans[chr(154)] = '&scaron;';    // Latin Small Letter S With Caron
					$trans[chr(155)] = '&rsaquo;';    // Single Right-Pointing Angle Quotation Mark
					$trans[chr(156)] = '&oelig;';    // Latin Small Ligature OE
					$trans[chr(159)] = '&Yuml;';    // Latin Capital Letter Y With Diaeresis
					$html = strtr($html, $trans);
				}
				$html = mb_convert_encoding($html, 'utf-8', $encoding);
			}
		}
	}
	return $html;
}

////////////////////////////////
// Check if service is enabled
////////////////////////////////
if (!$options->enabled) { 
	die('The full-text RSS service is currently disabled'); 
}

////////////////////////////////
// Check for feed URL
////////////////////////////////
if (!isset($_GET['url'])) { 
	die('No URL supplied'); 
}
$url = $_GET['url'];
if (!preg_match('!^https?://.+!i', $url)) {
	$url = 'http://'.$url;
}
$valid_url = filter_var($url, FILTER_VALIDATE_URL);
if ($valid_url !== false && $valid_url !== null && preg_match('!^https?://!', $valid_url)) {
	$url = filter_var($url, FILTER_SANITIZE_URL);
} else {
	die('Invalid URL supplied');
}

////////////////////////////////
// Redirect to alternative URL?
////////////////////////////////
if ($options->alternative_url != '' && !isset($_GET['redir']) && mt_rand(0, 100) > 50) {
	$redirect = $options->alternative_url.'?redir=true&url='.urlencode($url);
	if (isset($_GET['html'])) $redirect .= '&html='.urlencode($_GET['html']);	
	if (isset($_GET['key'])) $redirect .= '&key='.urlencode($_GET['key']);
	if (isset($_GET['max'])) $redirect .= '&max='.(int)$_GET['max'];
	header("Location: $redirect");
	exit;
}

///////////////////////////////////////////////
// Check if the request is explicitly for an HTML page
///////////////////////////////////////////////
$html_only = (isset($_GET['html']) && $_GET['html'] == 'true');

///////////////////////////////////////////////
// Check if valid key supplied
///////////////////////////////////////////////
$valid_key = (isset($_GET['key']) && in_array($_GET['key'], $options->api_keys));

///////////////////////////////////////////////
// Check URL against list of blacklisted URLs
// TODO: set up better system for this
///////////////////////////////////////////////
foreach ($options->blocked_urls as $blockurl) {
	if (strstr($url, $blockurl) !== false) {
		die('URL blocked');
	}
}

///////////////////////////////////////////////
// Max entries
// see config.php to find these values
///////////////////////////////////////////////
if ($valid_key && isset($_GET['max'])) {
	$max = (int)$_GET['max'];
	$max = min($max, $options->max_entries_with_key);
} else {
	$max = $options->max_entries;
}

////////////////////////////////
// Check for valid format
////////////////////////////////
$format = 'rss';

//////////////////////////////////
// Check for cached copy
//////////////////////////////////
$cache_file = 'cache/'.md5($max.$url.$valid_key).'.xml';
if (file_exists($cache_file)) {
	$cache_mtime = filemtime($cache_file);
	$diff = time() - $cache_mtime;
	$diff = $diff / 60;
	$diff_time = ($valid_key) ? 10 : 20; // 10 minutes cache for api key holders, 20 for others
	if ($diff < $diff_time) { // recent cache file exists, send that out and stop processing
		header("Content-type: text/xml; charset=UTF-8");
		if (headers_sent()) die('Some data has already been output to browser, can\'t send RSS file');
		readfile($cache_file);
		exit;
	}
}

//////////////////////////////////
// Set Expires header
//////////////////////////////////
if ($valid_key) {
	header('Expires: ' . gmdate('D, d M Y H:i:s', time()+(60*10)) . ' GMT');
} else {
	header('Expires: ' . gmdate('D, d M Y H:i:s', time()+(60*20)) . ' GMT');
}

//////////////////////////////////
// Delete old cached copies
//////////////////////////////////
if (mt_rand(0, 20) > 18) {
	// delete files older than $expire_time minutes
	$expire_time = 25; 
	// find all files of the given file type
	foreach (glob('cache/*.xml') as $filename) {
		// calculate file age in seconds
		$file_age = time() - @filemtime($filename);
		// is the file older than the given time span?
		if ($file_age > ($expire_time * 60)) {
			@unlink($filename);
		}
	}
	unset($expire_time, $filename, $file_age);
}

////////////////////////////////
// Get RSS/Atom feed
////////////////////////////////
if (!$html_only) {
	$feed = new SimplePie();
	$feed->set_feed_url($url);
	$feed->set_autodiscovery_level(SIMPLEPIE_LOCATOR_NONE);
	$feed->set_timeout(20);
	$feed->enable_cache(false);
	$feed->set_stupidly_fast(true);
	$feed->enable_order_by_date(false); // we don't want to do anything to the feed
	$feed->set_url_replacements(array());
	$result = $feed->init();
	//$feed->handle_content_type();
	//$feed->get_title();
	if ($result && (!is_array($feed->data) || count($feed->data) == 0)) {
		die('Sorry, no feed items found');
	}
}

////////////////////////////////////////////////////////////////////////////////
// Extract content from HTML (if URL is not feed or explicit HTML request has been made)
////////////////////////////////////////////////////////////////////////////////
if ($html_only || !$result) {
	$html = @file_get_contents($url);
	if (!$html) die('Error retrieving '.$url);
	$html = convert_to_utf8($html, $http_response_header);
	$node = grabArticle($html);
	$title = $node->firstChild->textContent;
	$content = $node->ownerDocument->saveXML($node->lastChild);
	if (!$valid_key) {
		$content = $options->message_to_prepend.$content;
		$content .= $options->message_to_append;
	} else {
		$content = $options->message_to_prepend_with_key.$content;	
		$content .= $options->message_to_append_with_key;
	}
	unset($node, $html);
	$output = new FeedWriter(); //ATOM an option
	$output->setTitle($title);
	$output->setDescription("Content extracted from $url");
	if ($format == 'atom') {
		$output->setChannelElement('updated', date(DATE_ATOM));
		$output->setChannelElement('author', array('name'=>'Five Filters', 'uri'=>'http://fivefilters.org'));
	}
	$output->setLink($url);
	$newitem = $output->createNewItem();
	$newitem->setTitle($title);
	$newitem->setLink($url);
	if ($format == 'atom') {
		$newitem->setDate(time());
		$newitem->addElement('content', $content);
	} else {
		$newitem->setDescription($content);
	}
	$output->addItem($newitem);
	$output->genarateFeed(); 
	exit;
}

////////////////////////////////////////////
// Create full-text feed
////////////////////////////////////////////

$output = new FeedWriter(); //ATOM an option
$output->setTitle($feed->get_title());
$output->setDescription($feed->get_description());
$output->setLink($feed->get_link());
if ($img_url = $feed->get_image_url()) {
	$output->setImage($feed->get_title(), $feed->get_link(), $img_url);
}
if ($format == 'atom') {
	$output->setChannelElement('updated', date(DATE_ATOM));
	$output->setChannelElement('author', array('name'=>'Five Filters', 'uri'=>'http://fivefilters.org'));
}

////////////////////////////////////////////
// Loop through feed items
////////////////////////////////////////////
$items = $feed->get_items(0, $max);	 
foreach ($items as $item) {
	// some URLs appear to have characters HTML encoded - does decoding affect other URLs?
	$permalink = htmlspecialchars_decode($item->get_permalink());
	$permalink = filter_var($permalink, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED);
	if ($permalink !== false && $permalink !== null && preg_match('!^https?://!', $permalink)) {
		$permalink = filter_var($permalink, FILTER_SANITIZE_URL);
	} else {
		$permalink = false;
	}
	$newitem = $output->createNewItem();
	$newitem->setTitle(htmlspecialchars_decode($item->get_title()));
	if ($permalink !== false) {
		$newitem->setLink($permalink);
	} else {
		$newitem->setLink($item->get_permalink());
	}
	
	if ($permalink && $html = @file_get_contents($permalink)) {
		$html = convert_to_utf8($html, $http_response_header);
		$html = grabArticleHtml($html, false);
		if (!$valid_key) {
			$html = $options->message_to_prepend.$html;
			$html .= $options->message_to_append;
		} else {
			$html = $options->message_to_prepend_with_key.$html;	
			$html .= $options->message_to_append_with_key;
		}
	} else {
		if (!$valid_key) {
			$html = $options->error_message;
		} else {
			$html = $options->error_message_with_key;
		}
		$html .= $item->get_description();
	}
	if ($format == 'atom') {
		$newitem->addElement('content', $html);
		$newitem->setDate((int)$item->get_date('U'));
		if ($author = $item->get_author()) {
			$newitem->addElement('author', array('name'=>$author->get_name()));
		}
	} else {
		$newitem->addElement('guid', $item->get_permalink(), array('isPermaLink'=>'true'));
		$newitem->setDescription($html);
		if ((int)$item->get_date('U') > 0) {
			$newitem->setDate((int)$item->get_date('U'));
		}
		if ($author = $item->get_author()) {
			$newitem->addElement('dc:creator', $author->get_name());
		}
	}
	$output->addItem($newitem);
	unset($html);
}
// output feed
ob_start();
$output->genarateFeed();
$output = ob_get_contents();
ob_end_clean();
file_put_contents($cache_file, $output);
echo $output;
?>